
Public Class Node(Of T)
    Public Property Item As T
    Public ReadOnly Property Neighbors As New List(Of Node(Of T))
    Public ReadOnly Property Weights As New List(Of Double?)

    Public Overrides Function ToString() As String
        Return Item?.ToString()
    End Function
End Class

Public Class Graph(Of T)
    Public ReadOnly Property Nodes As New List(Of Node(Of T))

    Public Function AddNode(item As T) As Node(Of T)
        Dim node As New Node(Of T) With {.Item = item}
        Nodes.Add(node)
        Return node
    End Function

    Public Sub AddEdge(fromNode As Node(Of T), toNode As Node(Of T), Optional weight As Double? = Nothing)
        fromNode.Neighbors.Add(toNode)
        fromNode.Weights.Add(weight)
        toNode.Neighbors.Add(fromNode)
        toNode.Weights.Add(weight)
    End Sub

    Public Overrides Function ToString() As String
        Dim s As String = ""

        Nodes.ForEach(
            Sub(node)
                s += $"{node} -> "

                Dim neighbors As New List(Of String)
                For i As Integer = 0 To node.Neighbors.Count - 1
                    Dim neigbor = node.Neighbors(i)
                    Dim weight = node.Weights(i)
                    neighbors.Add($"{neigbor} ({weight})")
                Next
                s += String.Join(", ", neighbors) & vbCrLf
            End Sub)
        Return s
    End Function
End Class