﻿Public Class LinkedList
    Implements IEnumerable

    Private NotInheritable Class Node
        Public Property Item As Object
        Public Property [Next] As Node
    End Class

    Public Property Count() As Integer
        Get
            Return _count
        End Get
        Private Set
            _count = Value
        End Set
    End Property
    Private _count As Integer

    Private first As Node, last As Node

    Default Public Property Item(index As Integer) As Object
        Get
            Return FindByIndex(index).Item
        End Get
        Set
            FindByIndex(index).Item = Value
        End Set
    End Property

    Public Sub Add(item As Object)
        Dim newNode = New Node() With {
            .Item = item,
            .Next = Nothing
        }

        If first Is Nothing Then
            first = newNode
            last = newNode
        Else
            last.Next = newNode ' verketteter Knoten
            last = newNode      ' neuer letzter Knoten
        End If
        Count += 1

    End Sub

    Public Function Contains(item As Object) As Boolean
        Dim node = Find(item)
        If node Is Nothing Then
            Return False
        End If
        Return True
    End Function

    Public Function Remove(item As Object) As Boolean
        Dim node As Node = Find(item)

        If node Is Nothing Then
            Return False
        End If

        Dim previousNode As Node = FindPrevious(item)

        If previousNode IsNot Nothing Then  ' aus Mitte oder Ende entfernen
            previousNode.Next = node.Next

            If Object.ReferenceEquals(node, last) = True Then
                last = previousNode
            End If
        Else
            first = node.Next               ' ersten entfernen, previousNode = null

            If first Is Nothing Then        ' Liste leer
                last = Nothing
            End If
        End If

        Count -= 1

        Return True
    End Function

    Public Sub Clear()
        first = Nothing
        last = Nothing
    End Sub



    Private Function FindPrevious(item As Object) As Node
        Dim previousNode As Node = Nothing
        Dim node As Node = first

        While node IsNot Nothing
            If node.Item.Equals(item) Then
                Return previousNode
            End If

            previousNode = node
            node = node.Next
        End While
        Return Nothing
    End Function

    Private Function Find(item As Object) As Node
        Dim node As Node = first

        While node IsNot Nothing
            If node.Item.Equals(item) Then
                Return node
            End If

            node = node.Next
        End While
        Return Nothing
    End Function

    Private Function FindByIndex(index As Integer) As Node
        If index >= Count Then
            Throw New IndexOutOfRangeException()
        End If

        Dim node As Node = first
        Dim i As Integer = 0

        While node IsNot Nothing
            If i = index Then
                Return node
            End If

            node = node.Next
            i += 1
        End While
        Return Nothing
    End Function

    'Public Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
    '    Return New LinkedListEnumerator(Me)
    'End Function
    Public Iterator Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
        Dim node As Node = first

        While node IsNot Nothing
            Yield node.Item
            node = node.[Next]
        End While
    End Function
    Public NotInheritable Class LinkedListEnumerator
        Implements IEnumerator

        Private list As LinkedList
        Private index As Integer

        Friend Sub New(list As LinkedList)
            Me.list = list
            index = -1
        End Sub

        Private ReadOnly Property Current() As Object Implements IEnumerator.Current
            Get
                Return _current
            End Get
        End Property
        Private _current As Object

        Private Function MoveNext() As Boolean Implements IEnumerator.MoveNext
            index += 1

            If index >= list.Count Then
                Return False
            End If

            _current = list(index)
            Return True
        End Function

        Private Sub Reset() Implements IEnumerator.Reset
            index = -1
        End Sub
    End Class
End Class

